@@ -38,6 +38,7 @@ def main():
     device_parser.add_argument('--access-token', action='store', help='Access token for device registration service. If not specified, taken from .roadtools_auth')
     device_parser.add_argument('--device-type', action='store', help='Device OS type (default: Windows)')
     device_parser.add_argument('--os-version', action='store', help='Device OS version (default: 10.0.19041.928)')
+    device_parser.add_argument('--deviceticket', action='store', help='Device MSA ticket to match with existing device')
 
     # Construct hybrid device module
     hdevice_parser = subparsers.add_parser('hybriddevice', help='Join an on-prem device to Azure AD')
@@ -271,6 +272,9 @@ def main():
     kdbauth_parser.add_argument('-k', '--keep-open',
                                 action='store_true',
                                 help='Do not close the browser window after timeout. Useful if you want to browse online apps with the obtained credentials')
+    kdbauth_parser.add_argument('--capture-code',
+                                action='store_true',
+                                help='Do not attempt to redeem any authentication code but print it instead')
 
     # Interactive auth using Selenium - inject PRT
     browserprtauth_parser = subparsers.add_parser('browserprtauth', help='Selenium based auth with automatic PRT usage. Emulates Edge browser with PRT')
@@ -299,6 +303,9 @@ def main():
     browserprtauth_parser.add_argument('--prt-sessionkey',
                                        action='store',
                                        help='Primary Refresh Token session key (as hex key)')
+    browserprtauth_parser.add_argument('--prt-cookie',
+                                       action='store',
+                                       help='Primary Refresh Token cookie from ROADtoken (JWT)')
     browserprtauth_parser.add_argument('--tokenfile',
                                        action='store',
                                        help='File to store the credentials (default: .roadtools_auth)',
@@ -313,6 +320,9 @@ def main():
     browserprtauth_parser.add_argument('-k', '--keep-open',
                                        action='store_true',
                                        help='Do not close the browser window after timeout. Useful if you want to browse online apps with the obtained credentials')
+    browserprtauth_parser.add_argument('--capture-code',
+                                       action='store_true',
+                                       help='Do not attempt to redeem any authentication code but print it instead')
 
     # Interactive auth using Selenium - inject PRT to other user
     injauth_parser = subparsers.add_parser('browserprtinject', help='Selenium based auth with automatic PRT injection. Can be used with other users to add device state to session')
@@ -425,7 +435,7 @@ def main():
                 jointype = 0
             else:
                 jointype = 4
-            deviceauth.register_device(tokenobject['accessToken'], jointype=jointype, certout=args.cert_pem, privout=args.key_pem, device_type=args.device_type, device_name=args.name, os_version=args.os_version)
+            deviceauth.register_device(tokenobject['accessToken'], jointype=jointype, certout=args.cert_pem, privout=args.key_pem, device_type=args.device_type, device_name=args.name, os_version=args.os_version, deviceticket=args.deviceticket)
         elif args.action == 'delete':
             if not deviceauth.loadcert(args.cert_pem, args.key_pem):
                 return
@@ -474,12 +484,16 @@ def main():
             return
         tokendata = deviceauth.aad_brokerplugin_prt_auth(args.client, args.resource, redirect_uri=args.redirect_url)
         # We need to convert this to a token format roadlib understands
-        tokenobject, _ = auth.parse_accesstoken(tokendata['access_token'])
-        tokenobject['expiresIn'] = tokendata['expires_in']
-        tokenobject['refreshToken'] = tokendata['refresh_token']
-        auth.outfile = args.tokenfile
-        auth.tokendata = tokenobject
-        auth.save_tokens(args)
+        if 'access_token' in tokendata:
+            tokenobject, _ = auth.parse_accesstoken(tokendata['access_token'])
+            tokenobject['expiresIn'] = tokendata['expires_in']
+            tokenobject['refreshToken'] = tokendata['refresh_token']
+            auth.outfile = args.tokenfile
+            auth.tokendata = tokenobject
+            auth.save_tokens(args)
+        else:
+            print('No access token in token data, assuming custom request')
+            print(tokendata)
     elif args.command == 'decrypt':
         header, enc_key, iv, ciphertext, auth_tag = auth.parse_compact_jwe(args.data, args.verbose)
         if header['alg'] == 'RSA-OAEP':
@@ -514,7 +528,7 @@ def main():
     elif args.command == 'codeauth':
         auth.set_client_id(args.client)
         auth.set_resource_uri(args.resource)
-        auth.authenticate_with_code_native(args.code, args.redirect_url)
+        auth.authenticate_with_code_native(args.code, args.redirect_url, client_secret=args.password)
         auth.outfile = args.tokenfile
         auth.save_tokens(args)
     elif args.command == 'listaliases':
@@ -555,7 +569,9 @@ def main():
         if not service:
             return
         selauth.driver = selauth.get_webdriver(service)
-        selauth.selenium_login(url, args.username, password, otpseed, keep=args.keep_open)
+        selauth.selenium_login(url, args.username, password, otpseed, keep=args.keep_open, capture=args.capture_code)
+        if args.capture_code:
+            return
         auth.outfile = args.tokenfile
         auth.save_tokens(args)
     elif args.command == 'browserprtauth':
@@ -563,6 +579,8 @@ def main():
         auth.set_resource_uri(args.resource)
         if args.prt and args.prt_sessionkey:
             deviceauth.setprt(args.prt, args.prt_sessionkey)
+        elif args.prt_cookie:
+            pass
         elif args.prt_file and deviceauth.loadprt(args.prt_file):
             pass
         else:
@@ -577,7 +595,9 @@ def main():
         if not service:
             return
         selauth.driver = selauth.get_webdriver(service, intercept=True)
-        if not selauth.selenium_login_with_prt(url, keep=args.keep_open):
+        if not selauth.selenium_login_with_prt(url, keep=args.keep_open, prtcookie=args.prt_cookie, capture=args.capture_code):
+            return
+        if args.capture_code:
             return
         auth.outfile = args.tokenfile
         auth.save_tokens(args)
